@@ -2,4 +2,4 @@
 #
 # vim: expandtab shiftwidth=4 softtabstop=4
 #
-from oc import *
\ No newline at end of file
+from .oc import *
\ No newline at end of file
