@@ -407,6 +407,7 @@ class Client(object):
         self.progress_class = kwargs.get('progress_class', progress.ProgressCR)
         self.oc_path_format = kwargs.get('oc_path_format', False)
         self.proxies = kwargs.get('proxies', None)
+        self.put_max_retry = kwargs.get('put_max_retry', 1)
 
     def login(self, user_id, password):
         """Authenticate to ownCloud.
@@ -572,7 +573,8 @@ class Client(object):
         path = self._normalize_path(path)
         res = self._session.get(
             self._webdav_url + parse.quote(self._encode_string(path)),
-            stream=True
+            stream=True,
+            headers={'User-Agent': 'gtocclient/{} ({}; {})'.format(pkg_info.version,getpass.getuser(),socket.gethostname())}
         )
         if res.status_code == 200:
             if not kwargs.get('dest_stream'):
@@ -603,7 +605,10 @@ class Client(object):
         debug(u'Download %s to %s' % (self._format_oc_path_for_output(remote_path), local_file))
         res = self._session.get(
             self._webdav_url + parse.quote(self._encode_string(remote_path)),
-            stream=True
+            stream=True,
+            headers={'User-Agent': 'gtocclient/{} ({}; {})'.format(pkg_info.version, getpass.getuser(),
+                                                                   socket.gethostname())}
+
         )
         if res.status_code == 200:
             if local_file is None:
@@ -649,7 +654,10 @@ class Client(object):
         remote_path = self._normalize_path(remote_path)
         url = self.url + 'index.php/apps/files/ajax/download.php?dir=' \
               + parse.quote(remote_path)
-        res = self._session.get(url, stream=True)
+        res = self._session.get(url,
+                                stream=True,
+                                headers={'User-Agent': 'gtocclient/{} ({}; {})'.format(pkg_info.version,getpass.getuser(),socket.gethostname())}
+)
         if res.status_code == 200:
             if local_file is None:
                 # use downloaded file name from Content-Disposition
@@ -678,7 +686,7 @@ class Client(object):
         """
         # if self.file_info(remote_path).file_type == "dir":
         #     raise Exception("远程路径为dir类型，无法上传")
-        mime_type = magic.from_buffer(data,mime=True)
+        mime_type = magic.from_buffer(data, mime=True)
         return self._make_dav_request('PUT', remote_path, data=data, headers={'Content-Type':mime_type}, **kwargs)
 
     def put_file(self, remote_path, local_source_file, **kwargs):
@@ -718,12 +726,24 @@ class Client(object):
             pass
 
         debug(u'Upload %s to %s' % (local_source_file, self._format_oc_path_for_output(remote_path)))
-        if kwargs.get('chunked', False) or kwargs.get('put_continue',False):
-            return self._put_file_chunked(
-                remote_path,
-                local_source_file,
-                **kwargs
-            )
+        retry = 0
+        if kwargs.get('chunked', False) or kwargs.get('put_continue', False):
+            while True:
+                try:
+                    result = self._put_file_chunked(
+                        remote_path,
+                        local_source_file,
+                        **kwargs
+                    )
+                    return result
+                except (HTTPResponseError, IOError) as e:
+                    retry += 1
+                    if retry >= self.put_max_retry:
+                        raise e
+                    else:
+                        print(" Error occurred: %s.\n Trying to upload the file again. Wait for 5 seconds..." % e)
+                        kwargs["put_continue"] = True
+                        time.sleep(5)
 
         stat_result = os.stat(local_source_file)
         if stat_result.st_size > 30*1024*1024*1024:
@@ -736,35 +756,43 @@ class Client(object):
         if kwargs.get('keep_mtime', True):
             headers['X-OC-MTIME'] = str(int(stat_result.st_mtime))
 
-        file_handle = open(local_source_file, 'rb', 8192)
-        if self.progress:
-            labels = {'source': local_source_file, 'destination': self._format_oc_path_for_output(remote_path), 'action': 'Upload',
-                      'extra': kwargs.get('extra_label') if kwargs.get('extra_label') else ""}
-            prog = self.progress_class(labels, os.path.getsize(local_source_file))
-            upload_iterator = utils.UploadIterator(file_handle, prog)
+        while True:
             try:
-                res = self._make_dav_request(
-                    'PUT',
-                    remote_path,
-                    data=utils.IterableToFileAdapter(upload_iterator),
-                    headers=headers
-                )
-                file_handle.close()
-                prog.done("done")
-                return res
-            except HTTPResponseError as e:
-                prog.done("failed")
-                raise e
-        else:
-            res = self._make_dav_request(
-                'PUT',
-                remote_path,
-                data=file_handle,
-                headers=headers
-            )
-            file_handle.close()
-            return res
-
+                file_handle = open(local_source_file, 'rb', 8192)
+                if self.progress:
+                    labels = {'source': local_source_file, 'destination': self._format_oc_path_for_output(remote_path), 'action': 'Upload',
+                              'extra': kwargs.get('extra_label') if kwargs.get('extra_label') else ""}
+                    prog = self.progress_class(labels, os.path.getsize(local_source_file))
+                    upload_iterator = utils.UploadIterator(file_handle, prog)
+                    try:
+                        res = self._make_dav_request(
+                            'PUT',
+                            remote_path,
+                            data=utils.IterableToFileAdapter(upload_iterator),
+                            headers=headers
+                        )
+                        file_handle.close()
+                        prog.done("done")
+                        return res
+                    except HTTPResponseError as e:
+                        prog.done("failed")
+                        raise e
+                else:
+                    res = self._make_dav_request(
+                        'PUT',
+                        remote_path,
+                        data=file_handle,
+                        headers=headers
+                    )
+                    file_handle.close()
+                    return res
+            except (HTTPResponseError, IOError) as e:
+                retry += 1
+                if retry >= self.put_max_retry:
+                    raise e
+                else:
+                    print(" Error occurred: %s.\n Trying to upload the file again. Wait for 5 seconds..." % e)
+                    time.sleep(5)
 
     def put_directory(self, target_path, local_directory, **kwargs):
         """Upload a directory with all its contents
